@@ -135,6 +135,7 @@ mark_appointment_changed(appt_win *apptw)
     if (!apptw->appointment_changed) {
         apptw->appointment_changed = TRUE;
         gtk_widget_set_sensitive(apptw->appRevert, TRUE);
+        gtk_widget_set_sensitive(apptw->appFileRevert_menuitem, TRUE);
     }
 }
 
@@ -144,6 +145,7 @@ mark_appointment_unchanged(appt_win *apptw)
     if (apptw->appointment_changed) {
         apptw->appointment_changed = FALSE;
         gtk_widget_set_sensitive(apptw->appRevert, FALSE);
+        gtk_widget_set_sensitive(apptw->appFileRevert_menuitem, FALSE);
     }
 }
 
@@ -432,6 +434,16 @@ on_appAllDay_clicked_cb(GtkCheckButton *checkbutton, gpointer user_data)
     mark_appointment_changed((appt_win *)user_data);
 }
 
+void free_apptw(appt_win *apptw)
+{
+    gtk_widget_destroy(apptw->appWindow);
+    gtk_object_destroy(GTK_OBJECT(apptw->appTooltips));
+    g_free(apptw->xf_uid);
+    g_free(apptw->par);
+    xfical_appt_free(apptw->appt);
+    g_free(apptw);
+}
+
 gboolean
 appWindow_check_and_close(appt_win *apptw)
 {
@@ -447,16 +459,11 @@ appWindow_check_and_close(appt_win *apptw)
              GTK_STOCK_NO, GTK_RESPONSE_CANCEL,
              NULL);
 
-        if (result == GTK_RESPONSE_ACCEPT) {
-            gtk_widget_destroy(apptw->appWindow);
-            g_free(apptw);
-        }
-    }
-    else{
-        gtk_widget_destroy(apptw->appWindow);
-        gtk_object_destroy(GTK_OBJECT(apptw->appTooltips));
-        g_free(apptw);
+        if (result == GTK_RESPONSE_ACCEPT)
+            free_apptw(apptw);
     }
+    else
+        free_apptw(apptw);
     return TRUE;
 }
 
@@ -501,6 +508,18 @@ fill_appt_from_apptw(appt_data *appt, appt_win *apptw)
     gchar starttime[6], endtime[6];
     gint i;
 
+    /* Next line is fix for bug 2811.
+     * We need to make sure spin buttons do not have values which are not
+     * yet updated = visible.
+     * gtk_spin_button_update call would do it, but it seems to cause
+     * crash if done in on_app_spin_button_changed_cb and here we should
+     * either do it for all spin fields, which takes too many lines of code.
+     * if (GTK_WIDGET_HAS_FOCUS(apptw->StartTime_spin_hh))
+     *      gtk_spin_button_update;
+     * So we just change the focus field and then spin button value gets set.
+     * It would be nice to not to have to change the field though.
+     */
+    gtk_widget_grab_focus(apptw->appTitle_entry);
     /*Get the title */
     appt->title = g_strdup(gtk_entry_get_text(
             (GtkEntry *)apptw->appTitle_entry));
@@ -524,8 +543,6 @@ fill_appt_from_apptw(appt_data *appt, appt_win *apptw)
     g_sprintf(appt->starttime, XFICAL_APPT_TIME_FORMAT
             , current_t.tm_year + 1900, current_t.tm_mon + 1, current_t.tm_mday
             , current_t.tm_hour, current_t.tm_min, 0);
-    appt->start_tz_loc = g_strdup(g_object_get_data(
-            G_OBJECT(apptw->appStartTimezone_button), "LOCATION_ENG"));
 
     /* Get the end date and time and timezone */
     current_t = display_to_year_month_day(gtk_button_get_label(
@@ -539,8 +556,6 @@ fill_appt_from_apptw(appt_data *appt, appt_win *apptw)
     g_sprintf(appt->endtime, XFICAL_APPT_TIME_FORMAT
             , current_t.tm_year + 1900, current_t.tm_mon + 1, current_t.tm_mday
             , current_t.tm_hour, current_t.tm_min, 0);
-    appt->end_tz_loc = g_strdup(g_object_get_data(
-            G_OBJECT(apptw->appEndTimezone_button), "LOCATION_ENG"));
 
     /* Get the duration */
     appt->use_duration = gtk_toggle_button_get_active(
@@ -614,7 +629,7 @@ fill_appt_from_apptw(appt_data *appt, appt_win *apptw)
                 GTK_BUTTON(apptw->appRecur_until_button)));
         g_sprintf(appt->recur_until, XFICAL_APPT_TIME_FORMAT
                 , current_t.tm_year + 1900, current_t.tm_mon + 1
-                , current_t.tm_mday, 0, 0, 0);
+                , current_t.tm_mday, 23, 59, 10);
     }
     else
         g_warning("fill_appt_from_apptw: coding error, illegal recurrence");
@@ -641,7 +656,7 @@ gboolean
 save_xfical_from_appt_win(appt_win *apptw)
 {
     gboolean ok = FALSE;
-    appt_data *appt = xfical_appt_alloc();
+    appt_data *appt = apptw->appt;
 
     if (fill_appt_from_apptw(appt, apptw)) {
         /* Here we try to save the event... */
@@ -652,6 +667,8 @@ save_xfical_from_appt_win(appt_win *apptw)
                 if (ok) {
                     apptw->appointment_add = FALSE;
                     gtk_widget_set_sensitive(apptw->appDuplicate, TRUE);
+                    gtk_widget_set_sensitive(apptw->appFileDuplicate_menuitem
+                            , TRUE);
                     g_message("Orage **: Added: %s", apptw->xf_uid);
                 }
                 else
@@ -671,12 +688,11 @@ save_xfical_from_appt_win(appt_win *apptw)
             apptw->appointment_new = FALSE;
             mark_appointment_unchanged(apptw);
             if (apptw->eventlist != NULL)
-                recreate_eventlist_win((eventlist_win *)apptw->eventlist);
+                refresh_eventlist_win((eventlist_win *)apptw->eventlist);
             xfcalendar_mark_appointments();
         }
     }
 
-    xfical_appt_free(appt);
     return (ok);
 }
 
@@ -695,10 +711,8 @@ on_appSave_clicked_cb(GtkButton *button, gpointer user_data)
 void
 save_xfical_from_appt_win_and_close(appt_win *apptw)
 {
-    if (save_xfical_from_appt_win(apptw)) {
-        gtk_widget_destroy(apptw->appWindow);
-        g_free(apptw);
-    }
+    if (save_xfical_from_appt_win(apptw))
+        free_apptw(apptw);
 }
 
 void 
@@ -741,13 +755,10 @@ delete_xfical_from_appt_win(appt_win *apptw)
             }
 
         if (apptw->eventlist != NULL)
-            recreate_eventlist_win((eventlist_win *)apptw->eventlist);
+            refresh_eventlist_win((eventlist_win *)apptw->eventlist);
         xfcalendar_mark_appointments();
 
-        gtk_widget_destroy(apptw->appWindow);
-        g_free(apptw->xf_uid);
-        g_free(apptw->par);
-        g_free(apptw);
+        free_apptw(apptw);
     }
 }
 
@@ -867,10 +878,10 @@ on_appStartEndDate_clicked_cb(GtkWidget *button, gpointer *user_data)
 }
 
 void
-on_appStartEndTimezone_clicked_cb(GtkWidget *button, gpointer *user_data)
+on_appTimezone_clicked_cb(GtkWidget *button,  appt_win *apptw
+        , gchar **tz)
 {
 #define MAX_AREA_LENGTH 20
-    appt_win *apptw = (appt_win *)user_data;
     GtkTreeStore *store;
     GtkTreeIter iter1, iter2;
     GtkWidget *tree;
@@ -878,7 +889,7 @@ on_appStartEndTimezone_clicked_cb(GtkWidget *button, gpointer *user_data)
     GtkTreeViewColumn *col;
     GtkWidget *window;
     GtkWidget *sw;
-    xfical_timezone_array tz;
+    xfical_timezone_array tz_a;
     int i, j, result;
     char area_old[MAX_AREA_LENGTH], *loc, *loc_eng, *loc_int;
     GtkTreeSelection *sel;
@@ -888,17 +899,17 @@ on_appStartEndTimezone_clicked_cb(GtkWidget *button, gpointer *user_data)
     /* enter data */
     store = gtk_tree_store_new(N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);
     strcpy(area_old, "S T a R T");
-    tz = xfical_get_timezones();
-    for (i=0; i < tz.count-2; i++) {
+    tz_a = xfical_get_timezones();
+    for (i=0; i < tz_a.count-2; i++) {
         /* first area */
-        if (! g_str_has_prefix(tz.city[i], area_old)) {
-            for (j=0; tz.city[i][j] != '/' && j < MAX_AREA_LENGTH; j++) {
-                area_old[j] = tz.city[i][j];
+        if (! g_str_has_prefix(tz_a.city[i], area_old)) {
+            for (j=0; tz_a.city[i][j] != '/' && j < MAX_AREA_LENGTH; j++) {
+                area_old[j] = tz_a.city[i][j];
             }
             if (j < MAX_AREA_LENGTH)
                 area_old[j] = 0;
             else
-                g_warning("on_appStartEndTimezone_clicked_cb: too long line in zones.tab %s", tz.city[i]);
+                g_warning("on_appStartEndTimezone_clicked_cb: too long line in zones.tab %s", tz_a.city[i]);
 
             gtk_tree_store_append(store, &iter1, NULL);
             gtk_tree_store_set(store, &iter1
@@ -909,8 +920,8 @@ on_appStartEndTimezone_clicked_cb(GtkWidget *button, gpointer *user_data)
         /* then city translated and in base form used internally */
         gtk_tree_store_append(store, &iter2, &iter1);
         gtk_tree_store_set(store, &iter2
-                , LOCATION, _(tz.city[i]) 
-                , LOCATION_ENG, tz.city[i] 
+                , LOCATION, _(tz_a.city[i]) 
+                , LOCATION_ENG, tz_a.city[i] 
                 , -1);
     }
          
@@ -955,9 +966,8 @@ on_appStartEndTimezone_clicked_cb(GtkWidget *button, gpointer *user_data)
                                 , &loc_eng, -1);
                     }
                 else {
-                    loc = g_strdup(gtk_button_get_label(GTK_BUTTON(button)));
-                    loc_eng = g_object_get_data(G_OBJECT(button)
-                            , "LOCATION_ENG");
+                    loc = g_strdup(_(*tz));
+                    loc_eng = g_strdup((*tz));
                 }
                 break;
             case 1:
@@ -969,8 +979,8 @@ on_appStartEndTimezone_clicked_cb(GtkWidget *button, gpointer *user_data)
                 loc_eng = g_strdup("floating");
                 break;
             default:
-                loc = g_strdup(gtk_button_get_label(GTK_BUTTON(button)));
-                loc_eng = g_object_get_data(G_OBJECT(button), "LOCATION_ENG");
+                loc = g_strdup(_(*tz));
+                loc_eng = g_strdup((*tz));
                 break;
         }
     } while (result == 0) ;
@@ -979,38 +989,39 @@ on_appStartEndTimezone_clicked_cb(GtkWidget *button, gpointer *user_data)
         mark_appointment_changed(apptw);
     }
     gtk_button_set_label(GTK_BUTTON(button), loc);
-    if ((loc_int = g_object_get_data(G_OBJECT(button), "LOCATION_ENG")))
-        g_free(loc_int);
-    loc_int = g_strdup(loc_eng);
-    g_object_set_data(G_OBJECT(button), "LOCATION_ENG", loc_int);
+    if (*tz)
+        g_free(*tz);
+    *tz = g_strdup(loc_eng);
     g_free(loc);
     g_free(loc_eng);
     gtk_widget_destroy(window);
 }
 
+static void on_appStartTimezone_clicked_cb(GtkWidget *button
+        , gpointer *user_data)
+{
+    appt_win *apptw = (appt_win *)user_data;
+    appt_data *appt;
+
+    appt = apptw->appt;
+    on_appTimezone_clicked_cb(button, apptw, &appt->start_tz_loc);
+}
+
+static void on_appEndTimezone_clicked_cb(GtkWidget *button
+        , gpointer *user_data)
+{
+    appt_win *apptw = (appt_win *)user_data;
+    appt_data *appt;
+
+    appt = apptw->appt;
+    on_appTimezone_clicked_cb(button, apptw, &appt->end_tz_loc);
+}
+
 void
 fill_appt_window_times(appt_win *apptw, appt_data *appt)
 {
     char *startdate_to_display, *enddate_to_display;
     int year, month, day, hours, minutes;
-    gchar *s_tz, *e_tz, *s_tze, *e_tze;
-
-    if (appt->start_tz_loc) {
-        s_tze = g_strdup(appt->start_tz_loc);
-        s_tz  = g_strdup(_(appt->start_tz_loc));
-    }
-    else { /* null = local = no timezone = floating */
-        s_tze = g_strdup("floating");
-        s_tz  = g_strdup(_("floating"));
-    }
-    if (appt->end_tz_loc) {
-        e_tze = g_strdup(appt->end_tz_loc);
-        e_tz  = g_strdup(_(appt->end_tz_loc));
-    }
-    else { /* null = local = no timezone = floating */
-        e_tze = g_strdup("floating");
-        e_tz  = g_strdup(_("floating"));
-    }
 
     /* all day ? */
     gtk_toggle_button_set_active(
@@ -1030,12 +1041,9 @@ fill_appt_window_times(appt_win *apptw, appt_data *appt)
             gtk_spin_button_set_value(
                 GTK_SPIN_BUTTON(apptw->appStartTime_spin_mm), (gdouble)minutes);
         }
-        if (s_tz) {
+        if (appt->start_tz_loc) {
             gtk_button_set_label(GTK_BUTTON(apptw->appStartTimezone_button)
-                    , s_tz);
-            g_free(s_tz);
-            g_object_set_data(G_OBJECT(apptw->appStartTimezone_button)
-                    , "LOCATION_ENG", s_tze);
+                    , _(appt->start_tz_loc));
         }
         else /* we should never get here */
             g_warning("fill_appt_window_times: s_tz is null");
@@ -1057,12 +1065,9 @@ fill_appt_window_times(appt_win *apptw, appt_data *appt)
             gtk_spin_button_set_value(
                 GTK_SPIN_BUTTON(apptw->appEndTime_spin_mm), (gdouble)minutes);
         }
-        if (e_tz) {
+        if (appt->end_tz_loc) {
             gtk_button_set_label(GTK_BUTTON(apptw->appEndTimezone_button)
-                    , e_tz);
-            g_free(e_tz);
-            g_object_set_data(G_OBJECT(apptw->appEndTimezone_button)
-                    , "LOCATION_ENG", e_tze);
+                    , _(appt->end_tz_loc));
         }
         else /* we should never get here */
             g_warning("fill_appt_window_times: e_tz is null");
@@ -1165,6 +1170,14 @@ fill_appt_window(appt_win *apptw, char *action, char *par)
         g_error("fill_appt_window: Appointment initialization failed");
         return;
     }
+    apptw->appt = appt; 
+    /* let's make sure timezones are never null. 
+     * Remember: null means use floating */
+    if (!appt->start_tz_loc)
+        appt->start_tz_loc = g_strdup("floating");
+    if (!appt->end_tz_loc)
+        appt->end_tz_loc = g_strdup("floating");
+
 
     /* first flags */
     apptw->xf_uid = g_strdup(appt->uid);
@@ -1191,6 +1204,8 @@ fill_appt_window(appt_win *apptw, char *action, char *par)
     /* we only want to enable duplication if we are working with an old
      * existing app (=not adding new) */
     gtk_widget_set_sensitive(apptw->appDuplicate, !apptw->appointment_add);
+    gtk_widget_set_sensitive(apptw->appFileDuplicate_menuitem
+            , !apptw->appointment_add);
 
     /* window title */
     gtk_window_set_title(GTK_WINDOW(apptw->appWindow)
@@ -1311,7 +1326,6 @@ fill_appt_window(appt_win *apptw, char *action, char *par)
         gtk_toggle_button_set_active(
                 GTK_TOGGLE_BUTTON(apptw->appRecur_feature_normal_rb), TRUE);
     }
-    xfical_appt_free(appt);
 
     set_time_sensitivity(apptw);
     set_repeat_sensitivity(apptw);
@@ -1846,10 +1860,10 @@ appt_win
             G_CALLBACK(on_app_spin_button_changed_cb), apptw);
 
     g_signal_connect((gpointer) apptw->appStartTimezone_button, "clicked",
-            G_CALLBACK(on_appStartEndTimezone_clicked_cb), apptw);
+            G_CALLBACK(on_appStartTimezone_clicked_cb), apptw);
 
     g_signal_connect((gpointer) apptw->appEndTimezone_button, "clicked",
-            G_CALLBACK(on_appStartEndTimezone_clicked_cb), apptw);
+            G_CALLBACK(on_appEndTimezone_clicked_cb), apptw);
 
     g_signal_connect((gpointer) apptw->appDur_checkbutton, "clicked",
             G_CALLBACK(appDur_checkbutton_clicked_cb), apptw);
